"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IncreaseProcess = void 0;
const dayjs_1 = __importDefault(require("dayjs"));
const index_1 = require("./index");
const concat_1 = require("../../concat");
const parsers_1 = require("../../parsers");
const readLcov_1 = require("../../utils/readLcov");
const utils_1 = require("../../utils");
const git_1 = require("../../utils/git");
class IncreaseProcess extends index_1.BaseProcess {
    constructor(lcovPath, opts = { stream: {} }) {
        super();
        this.opts = {
            stream: {},
        };
        if (typeof lcovPath === 'string') {
            this.lcovPath = [lcovPath];
        }
        else {
            this.lcovPath = lcovPath;
        }
        const startDay = dayjs_1.default().startOf('month').format('YYYY-MM-DD');
        this.opts.cwd = utils_1.getActualGitRepoRoot(opts.cwd);
        this.opts.since = opts.since || startDay;
        this.opts.stream.name = opts.stream.name || 'file';
        this.opts.stream.opts = opts.stream.opts;
        this.opts.output = opts.output;
        this.opts.hash = opts.hash;
    }
    exec() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.opts.hash) {
                this.baseDiffCommitInfo = yield this.getCommitInfoByHash(this.opts.hash);
            }
            if (!this.baseDiffCommitInfo) {
                this.baseDiffCommitInfo = yield this.getCommitInfoByLogSince();
            }
            if (!this.opts.cwd) {
                throw new Error('must pass a git dir');
            }
            const createInfo = yield this.getCreateInfo();
            yield this.getLcov();
            this.format();
            if (this.opts.output) {
                this.output({
                    data: this.formatData,
                    commit: this.baseDiffCommitInfo,
                    createInfo,
                });
            }
            return {
                data: this.formatData,
                commit: this.baseDiffCommitInfo,
                createInfo,
                gitRepoInfo: this.getGitRepoInfo()
            };
        });
    }
    getLcov() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield readLcov_1.getLcovFile(this.lcovPath);
            const increaseConcat = yield new concat_1.IncreaseConcat({
                cwd: this.opts.cwd,
                hash: (_a = this.baseDiffCommitInfo) === null || _a === void 0 ? void 0 : _a.hash,
            }).concat(...res);
            this.lcov = increaseConcat.getRes();
        });
    }
    getCommitInfoByLogSince() {
        return __awaiter(this, void 0, void 0, function* () {
            const subDate = dayjs_1.default(this.opts.since).subtract(1, 'day').format('YYYY-MM-DD');
            const res = yield new parsers_1.LogParser({
                repo: this.opts.cwd,
                before: subDate,
            }).run();
            return res[0];
        });
    }
    getCommitInfoByHash(hash) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield git_1.getGitRepoCommitInfoByHash(hash, this.opts.cwd);
            return result || undefined;
        });
    }
}
exports.IncreaseProcess = IncreaseProcess;
