"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IncreaseConcat = void 0;
const os_1 = __importDefault(require("os"));
const fullConcat_1 = require("./fullConcat");
const parsers_1 = require("../parsers");
const utils_1 = require("../utils");
class IncreaseConcat {
    constructor(opts = {}) {
        this.opts = {};
        this.res = {
            $: {
                linesCovered: 0,
                linesValid: 0,
            },
            detail: {},
        };
        this.diffData = [];
        this.opts.cwd = utils_1.getActualGitRepoRoot(opts.cwd);
        this.opts.hash = opts.hash;
    }
    concat(...lcov) {
        return __awaiter(this, void 0, void 0, function* () {
            this.getLcov(lcov);
            yield this.getDiff();
            this.format();
            return this;
        });
    }
    getRes() {
        return this.res;
    }
    getLcov(lcov) {
        this.res = new fullConcat_1.LcovConcat().concat(...lcov).getRes();
    }
    getDiff() {
        return __awaiter(this, void 0, void 0, function* () {
            this.diffData = yield new parsers_1.DiffParser(this.opts.hash, {
                cwd: this.opts.cwd,
            }).run();
            if (os_1.default.platform() === 'win32') {
                this.diffData = this.diffData.map((item) => {
                    return Object.assign(Object.assign({}, item), { newPath: item.newPath.replace(/\\/g, '/') });
                });
            }
        });
    }
    format() {
        this.res.$ = {
            linesCovered: 0,
            linesValid: 0,
        };
        Object.keys(this.res.detail).forEach((lcovItem) => {
            const info = this.res.detail[lcovItem];
            delete this.res.detail[lcovItem];
            this.diffData.forEach((diffItem) => {
                var _a;
                if (lcovItem.toLocaleLowerCase().includes((_a = diffItem.newPath) === null || _a === void 0 ? void 0 : _a.toLocaleLowerCase())) {
                    const temp = {
                        lineRate: 0,
                        linesCovered: 0,
                        linesValid: 0,
                        lines: [],
                    };
                    const diffInsertLineArr = [];
                    const lineNumberCheckMap = {};
                    diffItem.hunks.forEach((hunk) => {
                        hunk.changes.forEach((change) => {
                            if (!change.lineNumber) {
                                return;
                            }
                            const lineNumberCheckItem = lineNumberCheckMap[change.lineNumber] || {};
                            lineNumberCheckItem[change.type] = change.content;
                            lineNumberCheckMap[change.lineNumber] = lineNumberCheckItem;
                            if ((change.type === 'insert')) {
                                diffInsertLineArr.push(change.lineNumber);
                            }
                        });
                    });
                    const diffLineArr = [];
                    diffInsertLineArr.forEach((lineNumber) => {
                        const lineNumberCheckItem = lineNumberCheckMap[lineNumber];
                        if (!lineNumberCheckItem) {
                            return;
                        }
                        if (lineNumberCheckItem.insert && lineNumberCheckItem.delete && (lineNumberCheckItem.insert === lineNumberCheckItem.delete)) {
                            return;
                        }
                        diffLineArr.push(lineNumber);
                    });
                    const lcovLineHit = {};
                    const lcovLineArr = [];
                    const details = info.lines;
                    details.forEach((detail) => {
                        const { hits } = detail;
                        const { number } = detail;
                        lcovLineHit[number] = hits;
                        lcovLineArr.push(parseInt(number, 10));
                    });
                    const diffLineArrSet = new Set(diffLineArr);
                    const intersectArr = lcovLineArr.filter((x) => diffLineArrSet.has(x));
                    intersectArr.forEach((line) => {
                        this.res.$.linesCovered += lcovLineHit[line] > 0 ? 1 : 0;
                        temp.linesCovered += lcovLineHit[line] > 0 ? 1 : 0;
                        temp.lines.push({
                            number: line.toString(),
                            hits: lcovLineHit[line],
                        });
                    });
                    this.res.$.linesValid += intersectArr.length;
                    temp.linesValid += intersectArr.length;
                    temp.lineRate = temp.linesCovered / temp.linesValid;
                    this.res.detail[lcovItem] = temp;
                }
            });
        });
    }
}
exports.IncreaseConcat = IncreaseConcat;
