"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runBySpawn = exports.getGitRepoCommitInfoByHash = exports.getGitRepoFirstCommitInfo = exports.getGitRepoRootPath = exports.getGitRepoCurrentBranch = exports.getGitRepoRemoteUrl = void 0;
const child_process_1 = require("child_process");
function getGitRepoRemoteUrl(cwd) {
    try {
        const res = child_process_1.execSync(`git remote -v`, {
            cwd: cwd || process.cwd(),
        })
            .toString()
            .split('\n');
        const matchResult = res[0].trim().match(/[^\s]*\s+(.*)\s+\(.*/i);
        return matchResult && matchResult[1] || '';
    }
    catch (e) {
        return '';
    }
}
exports.getGitRepoRemoteUrl = getGitRepoRemoteUrl;
function getGitRepoCurrentBranch(cwd) {
    try {
        const res = child_process_1.execSync(`git symbolic-ref --short -q HEAD`, {
            cwd: cwd || process.cwd(),
        })
            .toString()
            .split('\n');
        return res[0];
    }
    catch (e) {
        return '';
    }
}
exports.getGitRepoCurrentBranch = getGitRepoCurrentBranch;
function getGitRepoRootPath(cwd) {
    try {
        const res = child_process_1.execSync(`git rev-parse --show-toplevel`, {
            cwd: cwd || process.cwd(),
        })
            .toString()
            .split('\n');
        return res[0];
    }
    catch (e) {
        return '';
    }
}
exports.getGitRepoRootPath = getGitRepoRootPath;
function getGitRepoFirstCommitInfo(cwd) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const runResult = yield runBySpawn('git log --reverse --pretty="%H!!!%h!!!%aN!!!%aE!!!%ad!!!%B"', [], {
                cwd: cwd || process.cwd(),
            });
            const res = runResult.toString().split('\n');
            const first = res[0].split('!!!');
            return {
                hash: first[0],
                abbrevHash: first[1],
                authorName: first[2],
                authorEmail: first[3],
                authorDate: first[4],
                subject: first[5],
            };
        }
        catch (e) {
            return null;
        }
    });
}
exports.getGitRepoFirstCommitInfo = getGitRepoFirstCommitInfo;
function getGitRepoCommitInfoByHash(hash, cwd) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const runResult = yield runBySpawn(`git log ${hash} --pretty="%H!!!%h!!!%aN!!!%aE!!!%ad!!!%B"`, [], {
                cwd: cwd || process.cwd(),
            });
            const res = runResult.toString().split('\n');
            const first = res[0].split('!!!');
            return {
                hash: first[0],
                abbrevHash: first[1],
                authorName: first[2],
                authorEmail: first[3],
                authorDate: first[4],
                subject: first[5],
            };
        }
        catch (e) {
            return null;
        }
    });
}
exports.getGitRepoCommitInfoByHash = getGitRepoCommitInfoByHash;
function runBySpawn(command, args, options) {
    return new Promise((resolve) => {
        const cmd = child_process_1.spawn(command, args, Object.assign(Object.assign({}, options), { shell: true }));
        let result = '';
        const checkT = setTimeout(() => {
            resolve(result);
        }, 5000);
        cmd.stdout.on('data', (data) => {
            result += data;
        });
        cmd.stderr.on('data', (data) => {
            console.error(`stderr: ${data}`);
        });
        cmd.on('close', (code) => {
            if (code) {
                console.error(`${command} close: ${code}`);
            }
            clearTimeout(checkT);
            resolve(result);
        });
    });
}
exports.runBySpawn = runBySpawn;
